@@ -26,7 +26,6 @@
 #include "notmuch-private.h"
 #include "database-private.h"
 
-#if HAVE_XAPIAN_FIELD_PROCESSOR
 static void
 compile_regex (regex_t &regexp, const char *str)
 {
@@ -124,12 +123,13 @@ bool
 RegexpPostingSource::check (Xapian::docid did, unused (double min_wt))
 {
     started_ = true;
-    if (!it_.check (did) || at_end ())
+    if (! it_.check (did) || at_end ())
 	return false;
     return (regexec (&regexp_, (*it_).c_str (), 0, NULL, 0) == 0);
 }
 
-static inline Xapian::valueno _find_slot (std::string prefix)
+static inline Xapian::valueno
+_find_slot (std::string prefix)
 {
     if (prefix == "from")
 	return NOTMUCH_VALUE_FROM;
@@ -145,11 +145,11 @@ RegexpFieldProcessor::RegexpFieldProcessor (std::string prefix,
 					    notmuch_field_flag_t options_,
 					    Xapian::QueryParser &parser_,
 					    notmuch_database_t *notmuch_)
-	: slot (_find_slot (prefix)),
-	  term_prefix (_find_prefix (prefix.c_str ())),
-	  options (options_),
-	  parser (parser_),
-	  notmuch (notmuch_)
+    : slot (_find_slot (prefix)),
+    term_prefix (_find_prefix (prefix.c_str ())),
+    options (options_),
+    parser (parser_),
+    notmuch (notmuch_)
 {
 };
 
@@ -158,17 +158,17 @@ RegexpFieldProcessor::operator() (const std::string & str)
 {
     if (str.empty ()) {
 	if (options & NOTMUCH_FIELD_PROBABILISTIC) {
-	    return Xapian::Query(Xapian::Query::OP_AND_NOT,
-			     Xapian::Query::MatchAll,
-			     Xapian::Query (Xapian::Query::OP_WILDCARD, term_prefix));
+	    return Xapian::Query (Xapian::Query::OP_AND_NOT,
+				  Xapian::Query::MatchAll,
+				  Xapian::Query (Xapian::Query::OP_WILDCARD, term_prefix));
 	} else {
 	    return Xapian::Query (term_prefix);
 	}
     }
 
     if (str.at (0) == '/') {
-	if (str.length() > 1 && str.at (str.size () - 1) == '/'){
-	    std::string regexp_str = str.substr(1,str.size () - 2);
+	if (str.length () > 1 && str.at (str.size () - 1) == '/') {
+	    std::string regexp_str = str.substr (1, str.size () - 2);
 	    if (slot != Xapian::BAD_VALUENO) {
 		RegexpPostingSource *postings = new RegexpPostingSource (slot, regexp_str);
 		return Xapian::Query (postings->release ());
@@ -176,14 +176,14 @@ RegexpFieldProcessor::operator() (const std::string & str)
 		std::vector<std::string> terms;
 		regex_t regexp;
 
-		compile_regex(regexp, regexp_str.c_str ());
+		compile_regex (regexp, regexp_str.c_str ());
 		for (Xapian::TermIterator it = notmuch->xapian_db->allterms_begin (term_prefix);
 		     it != notmuch->xapian_db->allterms_end (); ++it) {
-		    if (regexec (&regexp, (*it).c_str () + term_prefix.size(),
+		    if (regexec (&regexp, (*it).c_str () + term_prefix.size (),
 				 0, NULL, 0) == 0)
-			terms.push_back(*it);
+			terms.push_back (*it);
 		}
-		return Xapian::Query (Xapian::Query::OP_OR, terms.begin(), terms.end());
+		return Xapian::Query (Xapian::Query::OP_OR, terms.begin (), terms.end ());
 	    }
 	} else {
 	    throw Xapian::QueryParserError ("unmatched regex delimiter in '" + str + "'");
@@ -207,4 +207,3 @@ RegexpFieldProcessor::operator() (const std::string & str)
 	}
     }
 }
-#endif
